package org.vraptor.interceptor;

import org.apache.log4j.Logger;
import org.vraptor.Interceptor;
import org.vraptor.LogicException;
import org.vraptor.LogicFlow;
import org.vraptor.component.ComponentType;
import org.vraptor.component.LogicMethod;
import org.vraptor.introspector.Introspector;
import org.vraptor.reflection.GettingException;
import org.vraptor.view.ViewException;

/**
 * The last interceptor: calls the logic method.
 * 
 * @author Guilherme Silveira
 */
public class ExecuteLogicInterceptor implements Interceptor {

	private static final Logger LOG = Logger.getLogger(ExecuteLogicInterceptor.class);

	private ComponentType type;

	private LogicMethod logicMethod;

	private Introspector introspector;

	public ExecuteLogicInterceptor(Introspector introspector, ComponentType componentClass, LogicMethod logicMethod) {
		this.introspector = introspector;
		this.type = componentClass;
		this.logicMethod = logicMethod;
	}

	public void intercept(LogicFlow flow) throws LogicException, ViewException {
		Object component = flow.getLogicRequest().getLogicContext().getAttribute(
				ComponentLookupInterceptor.CURRENT_COMPONENT);

		if(LOG.isDebugEnabled()) {
			LOG.debug("executing business logic " + logicMethod);
		}
		
		String result = logicMethod.execute(component, flow.getLogicRequest());

		try {
			introspector.outject(type.getOutAnnotations(), component, flow.getLogicRequest());
		} catch (GettingException e) {
			throw new LogicException(e.getMessage(), e);
		}

		// TODO: an interceptor could have already set the result
		// should we check for it? since confirmResult throws exception when already called
		flow.getLogicRequest().confirmResult(result);
		flow.execute();
	}

}
